<?php
/*
* Opinus.pl
*
*  @author Opinus.pl <kontakt@opinus.pl>
*  @copyright  2019  Opinus.pl
*/


if(!defined('_PS_VERSION_'))
  exit;
 
class Opinus extends Module{
	private $_html='';
	
	public function __construct(){
		$this->name = 'opinus';
		$this->tab = 'advertising_marketing';
		$this->version = '1.0.1';
		$this->author = 'Opinus.pl';
		$this->need_instance = 0;
		$this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_); 
		$this->bootstrap = true;
		
		parent::__construct();
		
		$this->displayName = $this->l('Opinus');
		$this->description = $this->l('Collect and manage customers\' reviews.');
		
		$this->confirmUninstall = $this->l('Are you sure you want to uninstall? Remember - after uninstalling mode your customers will not receive any more invitations for writing reviews. Your data on Opinus.pl will not be removed after uninstalling mode.');
	}

	public function install(){
		if(Shop::isFeatureActive()){
			Shop::setContext(Shop::CONTEXT_ALL);
		}
		if(!parent::install() || !$this->registerHook('actionOrderStatusUpdate') || !Configuration::updateValue('OPINUS_DELAYDAYS', 1) || !Configuration::updateValue('OPINUS_ORDER_STATUS_SEND', 5)){
			return false;
		} else {
			return true;
		}
	}

	public function uninstall(){
		Configuration::deleteByName('OPINUS_KEY');
		Configuration::deleteByName('OPINUS_DELAYDAYS');
		Configuration::deleteByName('OPINUS_ORDER_STATUS_SEND');
		Configuration::deleteByName('OPINUS_ENV');
		if(!parent::uninstall()){
			return false;
		} else {
			return true;
		}
	}
	
	public function hookActionOrderStatusUpdate($params){
		$orderstatus2send_invitation=Configuration::get('OPINUS_ORDER_STATUS_SEND');
		$apikey=Configuration::get('OPINUS_KEY');
		$mode=Configuration::get('OPINUS_ENV');
		if(isset($orderstatus2send_invitation) && is_numeric($orderstatus2send_invitation) && $orderstatus2send_invitation>0 && $params['newOrderStatus']->id==$orderstatus2send_invitation && isset($apikey) && strlen($apikey)==20){
			if(isset($mode) && $mode===1){
				$api_url='https://opinus.pl/api/'; // https://opinus.pl/api/
			} else {
				$api_url=' https://opinus.pl/api-test/'; // https://opinus.pl/api-test/
			}
			
			$order_id=$params['id_order']; //dowolny unikatowy dla sklepu ciąg znaków, do 20 znaków, wymagane; tu licczbowa wartość
			$order=new Order($order_id);
			$customer_id=$order->id_customer;
			//$customer_id=$params['id_customer'];
			$customer=new Customer($customer_id); //dane klienta pobiera, tablicę!
			$customer_email=$customer->email; //adres e-mail klienta, wymagany, do wysłania zaproszenia do wystawienia opinii
			$customer_name=$customer->firstname; //imię klienta, opcjonalnie
			$customer_gender_id=$customer->id_gender; // płeć klienta, opcjonalnie, możliwe wartości: m, k
			if($customer_gender_id==1){
				$customer_gender='m';
			}
			if($customer_gender_id==2){
				$customer_gender='k';
			}
			$order_placedate=$order->date_add; //data złożenia zamówienia w formacie: YYYY-MM-DD HH:ii:ss, wymagana
			
			//przygotowanie zestawu danych do wysyłki i wywołanie żądania:
			$dataset = array(
				"authkey" => $apikey,
				"c_email" => $customer_email,
				"c_name" => $customer_name,
				"c_sex" => $customer_gender,
				"o_number" => $order_id,
				"o_placedate" => $order_placedate,
			);
			
			ob_start();
			$ch=curl_init($api_url);
			$data_json=json_encode($dataset);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $data_json);
			curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_0);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($ch, CURLOPT_FOLLOWLOCATION, false);
			
			$request_result=curl_exec($ch); //tu w odpowiedzi w formacie json status (wartość: 0 / 1) i treść odpowiedzi
			curl_close($ch);
			ob_end_flush();
			
			
			$answer=json_decode($request_result, true);
			if(isset($answer['status'])){
				if($answer['status']===1){
					//w tym momencie przy odpowiedzi poprawnej należy oznaczyć w systemie, że dane dla tego zamówienia już zostały wysłane do Opinus
					//wysłane poprawnie
					$response='wysłane poprawnie';
				} else {
					//błąd wysyłki
					$response='błąd wysyłki: '.$answer['msg'];
				}
			}
			
			//just debug:
			/*
			$myfile=fopen("test101.txt", "w");
			if(!isset($customer_email)){ $customer_email='brak'; }
			fwrite($myfile, 'Test: '.$response."\n".'order_id: '.$order_id.' '."\n".'email: '.$customer_email."\n".'imie: '.$customer_name."\n".'plec: '.$customer_gender."\n".'data: '.$order_placedate."\n");
			fclose($myfile);
			*/
		}
	}
	
	public function getContent(){
		$output1=null;
		$output2=null;
		if(Tools::isSubmit('submit'.$this->name.'api')){
			$op_api_key=strval(Tools::getValue('OPINUS_KEY'));
			if(isset($op_api_key) && !empty($op_api_key)){
				if(!Validate::isGenericName($op_api_key) || strlen($op_api_key)!=20){
					$output1.=$this->displayError($this->l('Invalid Opinus api key value'));
				} else {
					if($op_api_key!=Configuration::get('OPINUS_KEY')){
						Configuration::updateValue('OPINUS_KEY', $op_api_key);
						$output1.=$this->displayConfirmation($this->l('API key has been updated.'));
					}
				}
			}
			$op_environment=strval(Tools::getValue('OPINUS_ENV'));
			if(isset($op_environment)){
				if($op_environment!=0 && $op_environment!='' && $op_environment!=1){
					$output1.=$this->displayError($this->l('Incorrect environment type value.'));
				} else {
					Configuration::updateValue('OPINUS_ENV', $op_environment);
					$output1.=$this->displayConfirmation($this->l('Opinus mode has been updated.'));
				}
			}
		}
		if(Tools::isSubmit('submit'.$this->name.'settings')){
			$op_delay=strval(Tools::getValue('OPINUS_DELAYDAYS'));
			if(isset($op_delay)){
				if(!is_numeric($op_delay) || $op_delay<0){
					$output2.=$this->displayError($this->l('Incorrect number of delay (days).'));
				} else {
					if($op_delay!=Configuration::get('OPINUS_DELAYDAYS')){
						Configuration::updateValue('OPINUS_DELAYDAYS', $op_delay);
						$output2.=$this->displayConfirmation($this->l('Invitation delay time has been updated.'));
					}
				}
			}
			$op_orderstatus=strval(Tools::getValue('OPINUS_ORDER_STATUS_SEND'));
			if(isset($op_orderstatus)){
				if(!$op_orderstatus || empty($op_orderstatus) || !is_numeric($op_orderstatus) || $op_orderstatus<0){
					$output2.=$this->displayError($this->l('Invalid chosen order status.'));
				} else {
					if($op_orderstatus!=Configuration::get('OPINUS_ORDER_STATUS_SEND')){
						Configuration::updateValue('OPINUS_ORDER_STATUS_SEND', $op_orderstatus);
						$output2.=$this->displayConfirmation($this->l('Order status to send invitation for writing review has been updated.'));
					}
				}
			}
		}
		
		$this->_html .= $this->_showInfo();
		$this->_html .= $output1.$this->opinusApiKeyForm();
		$this->_html .= $output2.$this->opinusSettings();
		return $this->_html;
	}
	
	protected function _showInfo(){
		return $this->display(__FILE__, 'opinus-info.tpl');
	}
	
	public function opinusApiKeyForm(){
		//Get default language
		$default_lang = (int)Configuration::get('PS_LANG_DEFAULT');
		
		//Init Fields form array
		$fields_form[0]['form']=array(
			'legend' => array(
				'title' => $this->l('Communication with Opinus.pl'),
				'icon' => 'icon-cogs'
			),
			'input' => array(
				array(
					'type' => 'text',
					'label' => $this->l('Opinus.pl API key'),
					'desc' => $this->l('Opinus API key is available after siging in opinus.pl to managae your store. 20 chars.'),
					'name' => 'OPINUS_KEY',
					'required' => true,
					'maxlength' => 20,
					'placeholder' => 'abc123def456ghi789jk',
					'prefix' => '<i class="icon icon-key"></i>'
				),
				
				array(
					'type' => 'switch',
					'label' => $this->l('Production mode'),
					'name' => 'OPINUS_ENV',
					'required'  => true,
					'is_bool' => true,
					'desc' => $this->l('In test mode data is send to Opinus api but finally not save in portal.'),
					'values' => array(
						array(
							'id' => 'prd',
							'value' => 1,
							'label' => $this->l('production')
						),
						array(
							'id' => 'dev',
							'value' => 0,
							'label' => $this->l('test')
						)
					),
				),
				
			),
			'submit' => array(
				'title' => $this->l('Save')
			)
		);
		
		$helper1=new HelperForm();
		
		//Module, token and currentIndex
		$helper1->module = $this;
		$helper1->name_controller = $this->name;
		$helper1->token = Tools::getAdminTokenLite('AdminModules');
		$helper1->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
		
		//Language
		$helper1->default_form_language = $default_lang;
		$helper1->allow_employee_form_lang = $default_lang;
		
		//Title and toolbar
		$helper1->title = $this->displayName;
		$helper1->show_toolbar = true;        // false -> remove toolbar
		$helper1->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
		$helper1->submit_action = 'submit'.$this->name.'api';
		$helper1->toolbar_btn = array(
			'save' =>
			array(
				'desc' => $this->l('Save'),
				'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
				'&token='.Tools::getAdminTokenLite('AdminModules'),
			),
			'back' => array(
				'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
				'desc' => $this->l('Back to list')
			)
		);
		
		//Load current value
		$helper1->fields_value['OPINUS_KEY'] = Configuration::get('OPINUS_KEY');
		$helper1->fields_value['OPINUS_ENV'] = Configuration::get('OPINUS_ENV');
		
		return $helper1->generateForm($fields_form);
	}
	
	public function opinusSettings(){
		//Get default language
		$default_lang = (int)Configuration::get('PS_LANG_DEFAULT');
		
		$order_statuses=OrderState::getOrderStates((int)$this->context->language->id);
		$statuses[]=array('order_state_id'=>'0', 'order_state_name'=>'-- '.$this->l('choose status').' --');
		foreach($order_statuses as $status){
			//$statuses[$status['id_order_state']]=$status['name'];
			if($status['hidden']==0 && $status['deleted']==0){
				$statuses[]=array('order_state_id'=>$status['id_order_state'], 'order_state_name'=>$status['name'].' (id: '.$status['id_order_state'].')');
			}
		}
		
		//Init Fields form array
		$fields_form[0]['form']=array(
			'legend' => array(
				'title' => $this->l('Review invitations settings'),
				'icon' => 'icon-cogs'
			),
			'input' => array(
				array(
					'type' => 'select',
					'label' => $this->l('The status after which we want to send the invitation'),
					'desc' => $this->l('When order status will change to this status - then data will be send to Opinus portal to send invitation on next step.'),
					'name' => 'OPINUS_ORDER_STATUS_SEND',
					'options' => array(
						'query' =>  $statuses,
						'id' => 'order_state_id',
						'name' => 'order_state_name',
					),
					'required' => true,
					'prefix' => '<i class="icon icon-key"></i>'
				),
				array(
					'type' => 'text',
					'label' => $this->l('Delays (in days)'),
					'desc' => $this->l('Number of days counted from the moment when order status has changed to defined (for example "delivered"). After this time the invitation for writing review will be sent. 0 = no delay.'),
					'name' => 'OPINUS_DELAYDAYS',
					'required' => true,
					'maxlength' => 2,
					'prefix' => '<i class="icon icon-key"></i>'
				)
			),
			'submit' => array(
				'title' => $this->l('Save')
			)
		);
		$helper2=new HelperForm();
		
		if(!Configuration::get('OPINUS_ORDER_STATUS_SEND')){
			$helper2->fields_value['OPINUS_ORDER_STATUS_SEND']=5; //Ustawia wartość pola o danej nazwie. Tutaj domyślną wartość pola select
		} else {
			$helper2->fields_value['OPINUS_ORDER_STATUS_SEND']=Configuration::get('OPINUS_ORDER_STATUS_SEND');
		}
		if(!Configuration::get('OPINUS_DELAYDAYS')){
			$helper2->fields_value['OPINUS_DELAYDAYS']=0; //Ustawia wartość pola o danej nazwie gdy brak jeszcze skonfigurowanej. Tutaj domyślną wartość pola select
		} else {
			$helper2->fields_value['OPINUS_DELAYDAYS']=Configuration::get('OPINUS_DELAYDAYS');
		}
		
		//Module, token and currentIndex
		$helper2->module = $this;
		$helper2->name_controller=$this->name;
		$helper2->token = Tools::getAdminTokenLite('AdminModules');
		$helper2->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
		
		//Language
		$helper2->default_form_language = $default_lang;
		$helper2->allow_employee_form_lang = $default_lang;
		
		//Title and toolbar
		$helper2->title = $this->displayName;
		$helper2->show_toolbar = true;        // false -> remove toolbar
		$helper2->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
		$helper2->submit_action = 'submit'.$this->name.'settings';
		$helper2->toolbar_btn = array(
			'save' =>
			array(
				'desc' => $this->l('Save'),
				'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
				'&token='.Tools::getAdminTokenLite('AdminModules'),
			),
			'back' => array(
				'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
				'desc' => $this->l('Back to list')
			)
		);
		
		return $helper2->generateForm($fields_form);
	}

}

?>